@@ -2,9 +2,11 @@
 #define _HCK_LTRACE_H
 
 #include <sys/types.h>
+#include <sys/time.h>
 #include <stdio.h>
 
 #include "defs.h"
+#include "dict.h"
 
 /* BREAKPOINT_LENGTH is defined in "sysdep.h" */
 #include "sysdep.h"
@@ -69,6 +71,7 @@ struct callstack_element {
 	} c_un;
 	int is_syscall;
 	void * return_addr;
+	struct timeval time_spent;
 };
 
 #define MAX_CALLDEPTH 64
@@ -76,6 +79,7 @@ struct callstack_element {
 struct process {
 	char * filename;
 	pid_t pid;
+	struct dict * breakpoints;
 	int breakpoints_enabled;	/* -1:not enabled yet, 0:disabled, 1:enabled */
 
 	int callstack_depth;
@@ -114,6 +118,12 @@ struct event {
 	} e_un;
 };
 
+struct opt_c_struct {
+	int count;
+	struct timeval tv;
+};
+extern struct dict * dict_opt_c;
+
 extern struct process * list_of_processes;
 
 extern void * instruction_pointer;
@@ -123,12 +133,14 @@ extern void process_event(struct event * event);
 extern void execute_program(struct process *, char **);
 extern int display_arg(enum tof type, struct process * proc, int arg_num, enum arg_type at);
 extern struct breakpoint * address2bpstruct(struct process * proc, void * addr);
+extern void breakpoints_init(struct process * proc);
 extern void insert_breakpoint(struct process * proc, void * addr);
 extern void delete_breakpoint(struct process * proc, void * addr);
 extern void enable_all_breakpoints(struct process * proc);
 extern void disable_all_breakpoints(struct process * proc);
 extern struct process * open_program(char * filename);
 extern void open_pid(pid_t pid, int verbose);
+extern void show_summary(void);
 
 
 /* Arch-dependent stuff: */
@@ -137,6 +149,7 @@ extern void trace_me(void);
 extern int trace_pid(pid_t pid);
 extern void untrace_pid(pid_t pid);
 extern void * get_instruction_pointer(pid_t pid);
+extern void set_instruction_pointer(pid_t pid, void * addr);
 extern void * get_stack_pointer(pid_t pid);
 extern void * get_return_addr(pid_t pid, void * stack_pointer);
 extern void enable_breakpoint(pid_t pid, struct breakpoint * sbp);
